% expt2A.m -- Realistically eliminate distance pairs without noise.
%             Performs 25 trials and reports average normalized Frobenius
%             norm error.
%
%

% Number of trials.
T = 25;

% Vector to hold simulation parameters.
rmax = 0.1:0.01:1;
e = zeros(length(rmax), 1);    % Errors for non-squared distance matrix.
e2 = zeros(length(rmax), 1);   % Errors for squared distance matrix.
entryfrac = zeros(length(rmax), 1);  % Store average fraction of known entries.

% Do for all probabilities.
for (n = 1:1:length(rmax))
    % Do T trials.
    for (k = 1:1:T)       
        % Generate N random points inside the unit square.
        N = 200;
        X = rand(2, N);
        
        % Form Euclidean distance matrix.
        D = zeros(N, N);
        for (i = 1:1:N)
            for (j = 1:1:N)
                D(i, j) = norm(X(:, i) - X(:, j));
            end
        end

        % Convert to squares-of-distances matrix.
        D2 = D.^2;

        % Replace all zero entries (the ones on the diagonal) with epsilon.
        for (i = 1:1:N)
            D2(i, i) = eps;
        end

        % Knock out off-diagonal entries with distance larger than r_max.
        R = D2;
        
        for (i = 1:1:N)
            for (j = 1:1:N)
                if (D(i, j) > rmax(n))
                    R(i, j) = 0;
                end
            end
        end

        % Store fraction of known entries.
        entryfrac(n) = entryfrac(n) + (nnz(R) / (N * N));
        
        % Complete the matrix.  We know that it will be rank-4.
        [U, S, V, dist] = OptSpace(R, 4, 50, 1e-6);
        D2_hat = U*S*V';
        
        % Compute and store normalized Frobenius norm error for both
        % matrices.
        e(n) = e(n) + norm(D - sqrt(D2_hat), 'fro') / norm(D, 'fro');
        e2(n) = e2(n) + norm(D2 - D2_hat, 'fro') / norm(D2, 'fro');
    end
    
    % Average over T trials.
    e(n) = e(n) / T;
    e2(n) = e2(n) / T;
    entryfrac(n) = entryfrac(n) / T;
    
    % Print something to show that we're alive.
    disp(num2str(n));
end

% Plot error versus probability.
plot(rmax, e, 'b.');
title('Experiment 2A:  Average Frobenius Norm Error vs.  Maximum Radius');
xlabel('Maximum Radius');
ylabel('Average Frobenius Norm Error (Normalized)');

% Save data to file.
OUTPUT_FILE = 'EXPT2A.DAT';
FOUT = fopen(OUTPUT_FILE, 'wt');
for (i = 1:1:length(rmax))
    fprintf(FOUT, '%f ', rmax(i));
end
fprintf(FOUT, '\n\n');
for (i = 1:1:length(e))
    fprintf(FOUT, '%f ', e(i));
end
fprintf(FOUT, '\n\n');
for (i = 1:1:length(e2))
    fprintf(FOUT, '%f ', e2(i));
end
fprintf(FOUT, '\n\n');
for (i = 1:1:length(entryfrac))
    fprintf(FOUT, '%f ', entryfrac(i));
end